@@ -16,7 +16,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         setms <- funargs$setms
     }
     
-    
     if (!is.character(setms)) {
         
         if (inherits(testit <- tryCatch(eval(setms), error = function(e) e), "error")) {
@@ -29,7 +28,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
             
                 if (grepl("~", testit)) {
                 
-                    # try check if it's an object
                     if (eval.parent(parse(text=paste0("\"", gsub("~", "", testit), "\" %in% ls()")), n = 1)) {
                         setms <- 1 - eval.parent(parse(text=paste("get(\"", gsub("~", "", testit), "\")", sep="")), n = 1)
                         condnegated <- TRUE
@@ -44,7 +42,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
     else {
         if (setms == tolower(setms) & setms != toupper(setms)) {
             
-            # check if it's an object negated with lower case letters
             if (eval.parent(parse(text=paste0("\"", toupper(setms), "\" %in% ls()")), n = 1)) {
                 conds <- toupper(setms)
                 setms <- 1 - eval.parent(parse(text=paste("get(\"", toupper(setms), "\")", sep="")), n = 1)
@@ -53,7 +50,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         }
     }
     
-    
     outcomename <- ""
     outnegated <- NULL
     
@@ -61,8 +57,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         outcome <- funargs$outcome
     }
     
-    
-    
     if (!is.character(outcome)) {
         
         if (inherits(testit <- tryCatch(eval(outcome), error = function(e) e), "error")) {
@@ -75,7 +69,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
             
                 if (grepl("~", testit)) {
                 
-                    # try check if it's an object
                     if (eval.parent(parse(text=paste0("\"", gsub("~", "", testit), "\" %in% ls()")), n = 1)) {
                         outcome <- 1 - eval.parent(parse(text=paste("get(\"", gsub("~", "", testit), "\")", sep="")), n = 1)
                         condnegated <- TRUE
@@ -88,8 +81,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         }
     }
     
-    
-    # making sure the defaults are there
     icp <- 0.75
     ica <- 0.5
     
@@ -102,9 +93,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         ica <- incl.cut[2]
     }
     
-    ### 
-    ### ### backwards compatibility 
-    ### 
         neg.out <- FALSE
         if ("neg.out" %in% names(other.args)) {
             neg.out <- other.args$neg.out
@@ -117,9 +105,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         if ("incl.cut0" %in% names(other.args) & identical(ica, 0.5)) {
             ica <- other.args$incl.cut0
         }
-    ### 
-    ### ### backwards compatibility 
-    ### 
     
     recursive <- "recursive" %in% names(other.args)
     via.eqmcc <- "via.eqmcc" %in% names(other.args)
@@ -133,7 +118,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         condnegated <- identical(substr(gsub("[[:space:]]", "", funargs$setms), 1, 2), "1-")
     }
     
-    
     fuzzyop <- FALSE
         
     if (recursive) {
@@ -145,7 +129,7 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
     }
     else {
         
-        outcomename <- "Y" # a generic name in case nothing else is found
+        outcomename <- "Y" 
         
         if (!missing(data)) {
             
@@ -163,7 +147,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
             }
         }
         
-        
         if (class(setms) == "character") {
             if (length(setms) == 1) {
                 if (missing(data)) {
@@ -174,8 +157,7 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                 expression <- setms
                 
                 if (grepl("<=>", expression)) {
-                    # relation should be either necessity "<=" or sufficiency "=>"
-                    # but not both "<=>"
+                    
                     cat("\n")
                     stop(simpleError("Incorrect expression: relation can only be => or <=.\n\n"))
                 }
@@ -201,7 +183,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                     expression <- expression[1]
                 }
                 
-                
                 if (identical(outcome, "")) {
                     cat("\n")
                     stop(simpleError("Expression without outcome.\n\n"))
@@ -222,12 +203,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                         outcome <- substring(outcome, 2)
                     }
                     
-                    # if (! toupper(QCA::curlyBrackets(outcome, outside=TRUE)) %in% colnames(data)) {
-                    #     cat("\n")
-                    #     stop(simpleError("Inexisting outcome name.\n\n"))
-                    # }
-                    
-                    
                     if (grepl("\\{|\\}", outcome)) {
                         outcome.value <- QCA::curlyBrackets(outcome)
                         outcome <- QCA::curlyBrackets(outcome, outside=TRUE)
@@ -249,10 +224,16 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                 }
                 
                 if (is.character(outcome)) {
-                    setms <- compute(expression, data[, -which(colnames(data) == outcome)])
+                    setms <- compute(expression, data[, -which(colnames(data) == outcome)], separate = TRUE)
+                    if (ncol(setms) > 1) {
+                        setms$expression <- compute(expression, data[, -which(colnames(data) == outcome)])
+                    }
                 }
                 else {
-                    setms <- compute(expression, data)
+                    setms <- compute(expression, data, separate = TRUE)
+                    if (ncol(setms) > 1) {
+                        setms$expression <- compute(expression, data)
+                    }
                 }
                 
                 fuzzyop <- TRUE
@@ -263,10 +244,8 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
             }
         }
         
-        
         error <- FALSE
         
-        
         if (all(is.character(outcome)) & length(outcome) == 1) {
             if (missing(data)) {
                 cat("\n")
@@ -285,9 +264,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                     outcome <- gsub("1-", "", gsub("[[:space:]]|\"", "", funargs$outcome))
                 }
                 
-                
-                # for the moment, toupper(outcome) is redundant but if further on,
-                # the negation will be treated with lower case letters, it will prove important
                 if (! toupper(QCA::curlyBrackets(outcome, outside = TRUE)) %in% colnames(data)) {
                     cat("\n")
                     stop(simpleError("Inexisting outcome name.\n\n"))
@@ -299,12 +275,7 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                     
                     data[, toupper(outcome)] <- as.numeric(data[, toupper(outcome)] %in% QCA::splitstr(outcome.value))
                 }
-                ### this was supposed to treat the negation using lower case letters
-                # else if (! outcome %in% colnames(data)) {
-                #     data[, toupper(outcome)] <- 1 - data[, toupper(outcome)]
-                # }
                 
-                # the outcome was already converted to upper case letters
                 outcomename <- toupper(outcome)
                 outcome <- data[, outcomename]
             }
@@ -316,7 +287,7 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                 verify.qca(outcome)
                 
                 if (!inherits(tc <- tryCatch(QCA::getName(funargs$outcome), error = function(e) e), "error")) {
-                    # outcomename <- QCA::getName(funargs$outcome)
+                    
                     outcomename <- tc
                 }
             }
@@ -326,13 +297,11 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
             stop(simpleError("The outcome should be either a column name in a dataset\n       or a vector of set membership values.\n\n"))
         }
         
-        
         if (!(QCA::nec(relation) | QCA::suf(relation))) {
             cat("\n")
             stop(simpleError("The relationship should be either \"necessity\" or \"sufficiency\".\n\n"))
         }
         
-        
         if (!missing(data)) {
             
             if (length(outcome) != nrow(data)) {
@@ -340,7 +309,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                 stop(simpleError("The outcome's length should be the same as the number of rows in the data.\n\n"))
             }
             
-            
             if (any(outcomename %in% names(data))) {
                 noflevels <- truthTable(data, outcome=outcomename, via.pof=TRUE)
             }
@@ -351,7 +319,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
             
             colnames(data) <- toupper(colnames(data))
             
-            # this is redundant, just for visual clarity
             outcomename <- toupper(outcomename)
             conditions <- colnames(data)[-which(colnames(data) == outcomename)]
             data <- data[, c(conditions, outcomename)]
@@ -360,14 +327,13 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         pims <- FALSE
         
         if (class(setms) == "fuzzyop") {
-            # conditions <- attr(setms, "name")
+            
             conditions <- "expression"
             setms <- data.frame(X = as.vector(setms))
             colnames(setms) <- conditions
             fuzzyop <- TRUE
         }
         
-        
         if (is.data.frame(setms)) {
             
             if (missing(outcome)) {
@@ -376,8 +342,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
             }
             else {
                 
-                # do NOT uppercase these colnames, as they might be combinations of causal conditions
-                # either from pims or from fuzzyop
                 conditions <- colnames(setms)
                 
                 verify.qca(setms)
@@ -392,8 +356,7 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                     }
                 }
                 
-                
-                if (missing(data)) { # outcome was already checked to be (or coerced to) a vector
+                if (missing(data)) { 
                     if (nrow(setms) == length(outcome)) {
                         newdata <- cbind(setms, outcome)
                         colnames(newdata)[ncol(newdata)] <- outcomename
@@ -410,7 +373,7 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                     
                     if (nrow(setms) == nrow(data)) {
                         data <- cbind(setms, outcome)
-                        # colnames(data)[ncol(data)] <- toupper(outcomename)
+                        
                         pims <- TRUE
                     }
                     else {
@@ -471,7 +434,7 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                             stop(simpleError("Assuming this is a vector of row numbers, data argument is missing with no default (try force.rows = TRUE).\n\n"))
                         }
                         else {
-                            # setms %% 1 to check if it's a whole number (not integer, whole number!)
+                            
                             if (all(setms %% 1 == 0)) {
                                 cat("\n")
                                 stop(simpleError("Impossible to determine if the vector \"setms\" is data or row numbers.\n\n"))
@@ -481,16 +444,13 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                     
                     newdata <- cbind(setms, outcome)
                     
-                    #if (!fuzzyop) {
-                    conditions <- "X" # a generic name in case nothing else is found
-                    
+                    conditions <- "X" 
                     
                     if (!inherits(tc <- tryCatch(QCA::getName(funargs$setms), error = function(e) e), "error")) {
-                        # conditions <- QCA::getName(funargs$setms)
+                        
                         conditions <- tc
                     }
                     
-                    # colnames(newdata) <- toupper(c(conditions, outcomename))
                     colnames(newdata) <- c(conditions, outcomename)
                     pims <- TRUE
                     
@@ -516,8 +476,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
             noflevels <- truthTable(data, outcome=outcomename, via.pof=TRUE)
         }
         
-        # necessary here and not above because setms might be a vector
-        # and then translated into a matrix via getRow()
         if (is.matrix(setms)) {
             if (is.null(colnames(setms))) {
                 colnames(setms) <- toupper(conditions)
@@ -543,7 +501,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
             }
         }
         
-        
         if (!pims) {
             setms <- setms[, !hastime, drop=FALSE]
         }
@@ -551,7 +508,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         data[, which(hastime)] <- NULL
         conditions <- conditions[!hastime]
         
-        
         if (neg.out) {
             outcome <- 1 - outcome
         }
@@ -592,7 +548,7 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                 })
             })
             
-            if (!is.matrix(mins)) { ## bug fix 10.03.2014, if the data contains a single combination, mins is not a matrix but a vector
+            if (!is.matrix(mins)) { 
                 mins <- t(as.matrix(mins))
                 rownames(mins) <- rownames(data)
             }
@@ -604,9 +560,8 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         colnames(mins) <- conditions
     }
     
-    
     colnames(mins) <- gsub(", ", "", colnames(mins))
-    # if one is multivalue, all of them are
+    
     multivalue <- any(grepl("\\{|\\}", colnames(mins)))
     
     if (condnegated) {
@@ -614,17 +569,9 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         if (identical(conds, "")) {
             conds <- eval(parse(text = paste("attr(", gsub("1-", "", gsub("[[:space:]]", "", funargs$setms)), ", \"conditions\")")), envir=parent.frame())
         }
-        # if (multivalue) {
-            
-        # }
-        
-        
-        # parsed <- lapply(colnames(mins), function(x) translate(x, conds))
-        # return(parsed)
-        
         
         if (any(grepl("\\*", colnames(mins)))) {
-            # test if the object to be negated is a condition name
+            
             rownames(incl.cov) <- sapply(lapply(colnames(mins), deMorgan, use.tilde=any(grepl("~", colnames(mins))), prod.split="*"), function(x) {
                 return(paste(x[[1]][[2]], collapse="+"))
             })
@@ -643,15 +590,13 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
                 }
             }
             
-            
-            # test if the object to be negated is a product of single letter conditions
             if (all(toupper(unique(unlist(strsplit(colnames(mins), split="")))) %in% toupper(conds))) {
                 rownames(incl.cov) <- sapply(lapply(colnames(mins), deMorgan, use.tilde=any(grepl("~", colnames(mins)))), function(x) {
                     return(paste(x[[1]][[2]], collapse="+"))
                 })
             }
             else {
-                # cannot determine what it is, simply negate it with a tilde
+                
                 rownames(incl.cov) <- paste("~", colnames(mins))
             }
         }     
@@ -663,14 +608,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
     
     colnames(incl.cov) <- c("incl", "PRI", "cov.r", "cov.u")
     
-    
-    # for the moment, ron only but more can be added in the future
-    # optional.measures <- c("ron")
-    # optionals <- matrix(NA, nrow = nrow(incl.cov), ncol = length(optional.measures))
-    # colnames(optionals) <- optional.measures
-    # rownames(optionals) <- rownames(incl.cov)
-    
-    
     pmins <- apply(mins, 2, pmin, outcome)
     primins <- apply(mins, 2, function(x) pmin(x, 1 - outcome, outcome))
     
@@ -678,32 +615,28 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         primins <- apply(mins, 2, function(x) pmin(x, 1 - x, outcome))
     }
     
-    if (!is.matrix(pmins)) { ## bug fix 10.03.2014, if the data contains a single combination, pmins is not a matrix but a vector
+    if (!is.matrix(pmins)) { 
         pmins <- t(as.matrix(pmins))
         rownames(pmins) <- rownames(mins)
-                ## probably the very same thing happens to primins
+                
         primins <- t(as.matrix(primins))
         rownames(primins) <- rownames(mins)
     }
     
-    
     incl.cov[, 1] <- colSums(pmins)/colSums(mins)
     incl.cov[, 2] <- (colSums(pmins) - colSums(primins))/(colSums(mins) - colSums(primins))
     incl.cov[, 3] <- colSums(pmins)/sum.outcome
     
-    
     if (QCA::nec(relation)) {
         incl.cov[, 1] <- colSums(pmins)/sum.outcome
         
-        # TO REPLACE PRI with ron (PRI is only for sufficiency)
-        # incl.cov[, 2] <- (colSums(pmins) - colSums(primins))/(sum.outcome - colSums(primins))
         incl.cov[, 2] <- colSums(1 - mins)/colSums(1 - pmins)
         
         incl.cov[, 3] <- colSums(pmins)/colSums(mins)
-        # optionals[, "ron"] <- colSums(1 - mins)/colSums(1 - pmins)
+        
     }
     
-    maxmins <- unlist(fuzzyor(mins)) # union
+    maxmins <- unlist(fuzzyor(mins)) 
     inclusions <- unlist(fuzzyor(pmins))
     prisol <- pmin(maxmins, 1 - outcome, outcome)
     
@@ -711,7 +644,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         prisol <- pmin(maxmins, 1 - maxmins, outcome)
     }
     
-    
     if (ncol(mins) > 1) {
         if (fuzzyop) {
             pmins <- pmins[, seq(ncol(pmins) - 1)]
@@ -721,13 +653,10 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         }
     }
     
-    
-    # solution incl, pri and cov
     sol.incl <- sum(inclusions)/sum(maxmins)
     sol.pri <- (sum(inclusions) - sum(prisol))/(sum(maxmins) - sum(prisol))
     sum.cov <- sum(inclusions)/sum.outcome
     
-    
     result.list <- list(incl.cov=as.data.frame(incl.cov, stringsAsFactors = FALSE), relation=relation)
     
     if (!pims & via.eqmcc) {
@@ -739,7 +668,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         return(result.list)
     }
     
-    # showc is not a formal argument, therefore is it initiated as FALSE
     showc <- FALSE
     
     if (is.character(inf.test) & length(inf.test) == 1) {
@@ -755,7 +683,7 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         statistical.testing <- TRUE
         
         if (length(inf.test) > 1) {
-            alpha <- as.numeric(inf.test[2]) # already checked if a number between 0 and 1
+            alpha <- as.numeric(inf.test[2]) 
         }
         else {
             alpha <- 0.05
@@ -782,8 +710,6 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
         result.list$incl.cov <- incl.cov
     }
     
-    # incl.cov <- incl.cov[, c("incl", "cov.r", "cov.u", "PRI")]
-    
     if ("showc" %in% names(other.args)) {
         if (other.args$showc) {
             showc <- other.args$showc
@@ -810,7 +736,7 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
             colnames(result.list$incl.cov) <- cnames
         }
         result.list$options <- funargs[-1]
-        # result.list$optionals <- optionals
+        
         result.list$options$fuzzyop <- fuzzyop
         result.list$options$relation <- relation
         
@@ -818,5 +744,3 @@ function(setms, outcome, data, relation = "nec", inf.test = "",
     }   
 }
 
-
-
